@@ -44,7 +44,7 @@ def get_image_filename(self, code, format, options, prefix='nwdiag'):
     """
     Get path of output file.
     """
-    if format not in ('PNG', 'PDF'):
+    if format not in ('PNG', 'PDF', 'SVG'):
         raise NwdiagError('nwdiag error:\nunknown format: %s\n' % format)
 
     if format == 'PDF':
@@ -119,17 +119,48 @@ def create_nwdiag(self, code, format, filename, options, prefix='nwdiag'):
         screen = builder.ScreenNodeBuilder.build(tree)
 
         antialias = self.builder.config.nwdiag_antialias
-        draw = DiagramDraw.DiagramDraw(format, screen, filename, fontmap=fontmap,
-                                       antialias=antialias)
+        draw = DiagramDraw.DiagramDraw(format, screen, filename,
+                                       fontmap=fontmap, antialias=antialias)
     except Exception, e:
         raise NwdiagError('nwdiag error:\n%s\n' % e)
 
     return draw
 
 
+def make_svgtag(self, image, relfn, trelfn, outfn,
+                alt, thumb_size, image_size):
+    svgtag_format = """<svg xmlns="http://www.w3.org/2000/svg"
+    xmlns:xlink="http://www.w3.org/1999/xlink"
+    alt="%s" width="%s" height="%s">%s
+    </svg>"""
+
+    code = open(outfn, 'r').read().decode('utf-8')
+
+    return (svgtag_format %
+            (alt, image_size[0], image_size[1], code))
+
+
+def make_imgtag(self, image, relfn, trelfn, outfn,
+                alt, thumb_size, image_size):
+    result = ""
+    imgtag_format = '<img src="%s" alt="%s" width="%s" height="%s" />\n'
+
+    if trelfn:
+        result += ('<a href="%s">' % relfn)
+        result += (imgtag_format %
+                   (trelfn, alt, thumb_size[0], thumb_size[1]))
+        result += ('</a>')
+    else:
+        result += (imgtag_format %
+                   (relfn, alt, image_size[0], image_size[1]))
+
+    return result
+
+
 def render_dot_html(self, node, code, options, prefix='nwdiag',
                     imgcls=None, alt=None):
-    has_thumbnail = False
+    trelfn = None
+    thumb_size = None
     try:
         format = self.builder.config.nwdiag_html_image_format
         relfn, outfn = get_image_filename(self, code, format, options, prefix)
@@ -142,7 +173,6 @@ def render_dot_html(self, node, code, options, prefix='nwdiag',
         # generate thumbnails
         image_size = image.pagesize()
         if 'maxwidth' in options and options['maxwidth'] < image_size[0]:
-            has_thumbnail = True
             thumb_prefix = prefix + '_thumb'
             trelfn, toutfn = get_image_filename(self, code, format,
                                                 options, thumb_prefix)
@@ -170,15 +200,13 @@ def render_dot_html(self, node, code, options, prefix='nwdiag',
         if alt is None:
             alt = node.get('alt', self.encode(code).strip())
 
-        imgtag_format = '<img src="%s" alt="%s" width="%s" height="%s" />\n'
-        if has_thumbnail:
-            self.body.append('<a href="%s">' % relfn)
-            self.body.append(imgtag_format %
-                             (trelfn, alt, thumb_size[0], thumb_size[1]))
-            self.body.append('</a>')
+        if format == 'SVG':
+            tagfunc = make_svgtag
         else:
-            self.body.append(imgtag_format %
-                             (relfn, alt, image_size[0], image_size[1]))
+            tagfunc = make_imgtag
+
+        self.body.append(tagfunc(self, image, relfn, trelfn, outfn, alt,
+                                 thumb_size, image_size))
 
     self.body.append('</p>\n')
     raise nodes.SkipNode
@@ -212,10 +240,10 @@ def latex_visit_nwdiag(self, node):
 
 
 def on_doctree_resolved(self, doctree, docname):
-    if self.builder.name in ('html', 'latex'):
+    if self.builder.name in ('gettext', 'singlehtml', 'html', 'latex'):
         return
 
-    for node in doctree.traverse(nwdiag):  
+    for node in doctree.traverse(nwdiag):
         code = node['code']
         prefix = 'nwdiag'
         format = 'PNG'
@@ -227,7 +255,8 @@ def on_doctree_resolved(self, doctree, docname):
             image.draw()
             image.save()
 
-        image = nodes.image(uri=outfn)
+        candidates = {'image/png': outfn}
+        image = nodes.image(uri=outfn, candidates=candidates)
         node.parent.replace(node, image)
 
 
